/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#include <stdio.h>
#include <time.h>
#include <errno.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>

#include "nomadii.h"
#include <nomadII.h>

extern char* prog_name;

void usage()
{
	fprintf(stdout,"Usage: %s [options] [command]\n", prog_name);
	fprintf(stdout,"\tOPTIONS:\n");
	fprintf(stdout,"\t   -a, --autorename      : turn off auto-rename on list\n");
	fprintf(stdout,"\t   -v, --verbose         : verbose execution\n");
	fprintf(stdout,"\t   -b, --bell            : sounds indicating command completion\n");
	fprintf(stdout,"\t   -H, --hash            : display file transfer progress\n");
	fprintf(stdout,"\t   -t, --prompt          : interactive confirmation for --playlist\n");
	fprintf(stdout,"\t   -F, --force           : no confirmation for format/delete commands\n");
	fprintf(stdout,"\t   -o, --offset <n>      : playlist offset for sending files\n");
	fprintf(stdout,"\t   -n, --number <n>      : send <n> files from playlist\n");
	fprintf(stdout,"\t   -r, --replace         : delete all before sending files\n");
	fprintf(stdout,"\t       --freq   <f1f2>   : FM frequency f1.f2 for --presets-set\n");
	fprintf(stdout,"\tCOMMANDS:\n");
	fprintf(stdout,"\t   -h, --help            : help (usage reference)\n");
	fprintf(stdout,"\t   -V, --version         : version information\n");
	fprintf(stdout,"\t   -P, --probe           : probe USB bus for Nomad II device\n");
	fprintf(stdout,"\t   -M, --model           : display model and firmware information\n");
	fprintf(stdout,"\t   -m, --memory          : display memory information\n");
	fprintf(stdout,"\t   -c, --count           : display number of files on device\n");
	fprintf(stdout,"\t   -l, --list            : list files information\n");
	fprintf(stdout,"\t   -f, --format          : format flash memory\n");
	fprintf(stdout,"\t   -D, --delete   <n> .. : delete <n> ... or \"all\" files on Nomad II\n");
	fprintf(stdout,"\t   -s, --send     <f> .. : send files <f> ... to device\n");
	fprintf(stdout,"\t       --put      <f> .. : put files <f> ... on to device\n");
	fprintf(stdout,"\t   -d, --download <f> .. : download files <f> ... to device\n");
	fprintf(stdout,"\t   -p, --playlist <f>    : send files in playlist <f> to device\n");
	fprintf(stdout,"\t   -g, --get      <n> .. : get <n> ... or \"all\" files from device\n");
	fprintf(stdout,"\t   -u, --upload   <n> .. : upload <n> ... or \"all\" files from device\n");
	fprintf(stdout,"\t   -i  --internal        : use internal flash memory\n");
	fprintf(stdout,"\t   -x  --external        : use external flash memory\n");
	fprintf(stdout,"\t       --clock           : display device clock information\n");
	fprintf(stdout,"\t       --clock-set       : set device clock to current time\n");
	fprintf(stdout,"\t       --presets         : display FM preset information\n");
	fprintf(stdout,"\t       --presets-set <n> : set FM preset <n> to --freq value\n");
	fprintf(stdout,"\t       --owner           : display owner information\n");
	fprintf(stdout,"\t       --owner-set   <s> : set owner information\n");
#ifdef PACKET_DUMP
	fprintf(stdout,"\t       --debug           : increase debug level (2 == full dump)\n");
#endif 
}

/* 
 * Command Table 
 */
extern struct cmd cmdtab[];
extern struct cmd *getcmd();

#define HELPINDENT (sizeof ("directory"))

/*
 * Help command.
 */
int
help(int argc, char *argv[])
{
	register struct cmd *c;

	if (argc == 1) {
		register int i, j, w;
		int columns, width = 0, lines;
		extern int NCMDS;

		fprintf(stdout,"Commands may be abbreviated.  Commands are:\n\n");
		for (c = cmdtab; c->c_name; c++) {
			int len = strlen(c->c_name);

			if (len > width)
				width = len;
		}
		width = (width + 8) &~ 7;
		columns = 80 / width;
		if (columns == 0)
			columns = 1;
		lines = (NCMDS + columns - 1) / columns;
		for (i = 0; i < lines; i++) {
			for (j = 0; j < columns; j++) {
				c = cmdtab + j * lines + i;
				if (c->c_name)
					fprintf(stdout,"%s", c->c_name);
				if (c + lines >= &cmdtab[NCMDS]) {
					fprintf(stdout,"\n");
					break;
				}
				w = strlen(c->c_name);
				while (w < width) {
					w = (w + 8) &~ 7;
					putchar('\t');
				}
			}
		}
		return 0;
	}
	while (--argc > 0) {
		register char *arg;
		arg = *++argv;
		c = getcmd(arg);
		if (c == (struct cmd *)-1)
			fprintf(stderr,"?Ambiguous help command %s\n", arg);
		else if (c == (struct cmd *)0)
			fprintf(stderr,"?Invalid help command %s\n", arg);
		else
			fprintf(stdout,"%-*s\t%s\n", (int)HELPINDENT,
				c->c_name, c->c_help);
	}
	return 0;
}

void verb(char *msg) 
{
	if (verbose) fprintf(stdout,"%s",msg);
}

static const char *onoff(int bool)
{
        return (bool ? "on" : "off");
}

int version()
{
	fprintf(stdout,"nomadii V%s, Copyright 2002 Gabriel Gomiz <gomita@bblanca.com.ar>\n", VERSION);
	return 0;
}

int quit(int argc, char *argv[])
{
	if (nomadII.nomadII_isopen) {
	  verb("Closing NomadII...\n");
		dev_close();
	}
	exit(0);
	return 0;
}

int status()
{
	if (nomadII.nomadII_isopen)
		fprintf(stdout,"Device connected.\n");
	else
		fprintf(stdout,"Not connected.\n");

	fprintf(stdout,"Verbose: %s; Bell: %s; Prompting: %s; ", onoff(verbose), onoff(bell), onoff(prompt));
#ifdef PACKET_DUMP
        fprintf(stdout,"Autorename: %s, Hash marks: %s; Force: %s; Debug: %s\n", onoff(autorename), onoff(hash), onoff(force), onoff(debug));
#else
	fprintf(stdout,"Autorename: %s, Hash marks: %s; Force: %s\n", onoff(autorename), onoff(hash), onoff(force));
#endif
	if (!nomadII.both_present)
		return 0;

	switch (memory_bank) {
		case INTERNAL: 
			fprintf(stdout,"Using internal memory.\n");
			break;
		case EXTERNAL:
			fprintf(stdout,"Using external memory.\n");
			break;
		default:
			fprintf(stdout,"Using internal and external memory.\n");
			break;
	}
	return 0;
}

int memsync()
{
	if (nomadII_sync(&nomadII) != 0) {
		fprintf(stderr,"Error Unable to get in sync with device!\n");
		fprintf(stderr,"Nomad: %s\n", nomadII_get_error());
		return -1;
	}

	fprintf(stdout,"Synced OK.\n");
	return 0;
}

int dev_probe_status(int silent)
{
	if (!nomadII.found) {
		fprintf(stderr,"Error: Unable to find any Creative Nomad II devices\n");
		return -1;
	}

	if (!silent) {
		if (nomadII.prod_id == 0x1002)
			fprintf(stdout,"Creative Nomad II successfully found\n");
		else if (nomadII.prod_id == 0x4004)
			fprintf(stdout,"Creative Nomad II MG successfully found\n");
	}

	if (verbose) {
		fprintf(stdout,"Vendor = 0x%04x\tProduct = 0x%04x\n",nomadII.vendor_id,nomadII.prod_id);
		fprintf(stdout,"Bus    = %d\tDev = %d\n",nomadII.bus,nomadII.dev);
	}

	return 0;
}

int dev_reprobe()
{
	nomadII.found = FALSE;

	if (nomadII_probe(&nomadII) == 0)
		nomadII.found = TRUE;
		
	dev_probe_status(FALSE);
	return 0;
}


void dev_probe()
{
	uid_t uid  = getuid();
	uid_t euid = geteuid();

	nomadII.found = FALSE;

	if (nomadII_probe(&nomadII) == 0)
		nomadII.found = TRUE;

	if (euid == 0 && uid == 0) {
		fprintf(stderr, "Warning: it is not recommended that you run this program as root!\n");
	} else if (euid == 0 && uid != euid) {
#ifdef WITH_USBDEVFS
		/* change ownership of device to real user */
		if (nomadII.found && chown(nomadII.dst_dev, uid, -1) == -1)
			fprintf(stderr,"error changing ownership of %s: %s\n", nomadII.dst_dev,
				strerror(errno));
#endif
		if (setuid(uid) == -1) {
			fprintf(stderr,"error shedding setuid state: %s\n", strerror(errno));
			exit(1);
		}
	}
}

int dev_open()
{
	if (nomadII_open(&nomadII) != 0) {
		fprintf(stderr,"Error: NomadII could not be accessed: %s\n", nomadII_get_error());
		return -1;
	}

	if (nomadII_prepare(&nomadII) != 0) {
		fprintf(stderr,"Error: Unable to prepare device: %s\n", nomadII_get_error());
		return -1;
	}

	if (nomadII_sync(&nomadII) != 0) {
		fprintf(stderr,"Error: Unable to get in sync with device: %s\n", nomadII_get_error());
		return -1;
	}

	if (!nomadII.both_present) {
		if (memory_bank == INTERNAL && !nomadII.present[INTERNAL]) {
			fprintf(stderr,"Error: Internal specified and no internal memory!\n");
			return -1;
		} else if (memory_bank == EXTERNAL && !nomadII.present[EXTERNAL]) {
			fprintf(stderr,"Error: External specified and no internal memory!\n");
			return -1;
		} else if (memory_bank == BOTH) {
			memory_bank = nomadII.present[INTERNAL] ? INTERNAL : EXTERNAL;
		}
	}

	verb("Nomad II succesfully opened.\n");

	return 0;
}

int dev_close()
{
	if (nomadII_close(&nomadII) != 0) {
		fprintf(stderr,"Error: Nomad II could not be closed!\n");
		return -1;
	}

	verb("NomadII succesfully closed.\n");

	return 0;
}

int model()
{
	if (interactive)
		if (nomadII_model(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get NomadII model information!\n");
			return -1;
		}
	fprintf(stdout,"Model: %s\n",nomadII.model_info);
	fprintf(stdout,"Firmware: V%s\n",nomadII.fw_info);

	return 0;
}

int get_clock()
{
	if (interactive)
		if (nomadII_get_clock(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get NomadII device clock!\n");
			return -1;
		}

	fprintf(stdout,"Device clock: %s",asctime(&nomadII.dclock));

	return 0;
}

int set_clock()
{
	if (nomadII_set_clock(&nomadII) != 0) {
		fprintf(stderr,"Error: Could not set NomadII device clock!\n");
		return -1;
	}
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
	verb("Device clock written.\n");

	return 0;
}

int dev_clock(int argc, char *argv[])
{
	switch (argc) {
		
	case 1:	return get_clock();
		
	case 2:		  
		if (!strcmp(argv[1],"get")) {			
			return get_clock();
		}
		else if (!strcmp(argv[1],"set")) {
			return set_clock();
		}
		else fprintf(stderr,"?Invalid command\n");
		break;
	default:
		fprintf(stderr,"?Invalid command\n");
		
	}

	return -1;	
}

int get_owner()
{
	if (interactive)
		if (nomadII_get_owner(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get NomadII owner information!\n");
			return -1;
		}
	fprintf(stdout,"Owner: %s\n",nomadII.owner_info);
	return 0;
}

int set_owner(char *owner)
{
	if (strlen(owner) > MAX_OWNER_LENGTH) {
		char buff[10];
		strncpy(buff, owner, 9);
		buff[9] = '\0';
		fprintf(stderr,"Warning: owner will probably be truncated to: %s\n", buff);
	}

	strncpy(nomadII.owner_info,owner,STRING_SIZE);
	nomadII.owner_info[STRING_SIZE-1] = '\0';
	if (nomadII_set_owner(&nomadII) != 0) {
		fprintf(stderr,"Error: Could not set NomadII owner information!\n");
		return -1;
	}
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
	verb("Owner info written.\n");
	return 0;
}

int owner(int argc, char *argv[])
{
	switch (argc) {
		
	case 1:	return get_owner();
		
	case 2:		  
		if (!strcmp(argv[1],"get")) {			
			return get_owner();
		}
		else fprintf(stderr,"?Invalid command\n");
		break;

	default:
		if (strcmp(argv[1],"set") == 0 && argc == 3) {
			return set_owner(argv[2]);
		} else {
			fprintf(stderr,"?Invalid command\n");
		}
	}

	return -1;	
}

static void memory_print(char* label, int total, int free)
{
    fprintf(stdout,"%s: Size %.1fM, Used %.1fM, Free %.1fM\n", label, 
		    total/MBF,(total-free)/MBF, free/MBF);
}


int memory()
{
	char *label = "Memory";

	if (interactive)
		if (nomadII_memory(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get NomadII memory information!\n");
			return -1;
		}
	if (memory_bank != EXTERNAL && nomadII.present[INTERNAL]) {
		if (nomadII.both_present)
			label = "Internal";
		memory_print(label, nomadII.mem_total[INTERNAL], nomadII.mem_free[INTERNAL]);
	}
	if (memory_bank != INTERNAL && nomadII.present[EXTERNAL]) {
		if (nomadII.both_present)
			label = "External";
		memory_print(label, nomadII.mem_total[EXTERNAL], nomadII.mem_free[EXTERNAL]);		
	}
	return 0;
}

int count()
{
	int   count = 0;
	char *label = "";
	if (interactive) {
		if (nomadII_count_files(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get NomadII flash memory files information!\n");
			return -1;
		}
		if (nomadII_get_files(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get NomadII flash memory files information!\n");
			return -1;
		}
	}
	switch(memory_bank) {
		case INTERNAL : 
			count = nomadII.bank_count[INTERNAL]; 
			label = nomadII.both_present ? "internal " : "";
			break;
		case EXTERNAL : 
			count = nomadII.bank_count[EXTERNAL];
			label = nomadII.both_present ? "external " : "";
			break;
		case BOTH     : 
			count = nomadII.file_count;
			label = "internal and external ";
			break;
	}
	
	fprintf(stdout,"# of %sfiles: %d\n",label,count);
	return 0;
}


static void list_print(int i)
{
	char  filename[PATH_MAX];
	char  buff[5];
	char* format;
	char* bankmark;
	char* units;
	float size;
	nomadII_file *file = &nomadII.files[i];

	if (strlen(file->long_file_name) < 0)
		format = "%s%-4s %-62s (%.1f%s)\n";
	else
		format = "%s%-4s %s  (%.1f%s)\n";
	
	strncpy(filename, file->long_file_name, PATH_MAX - 1);
	
	if (autorename) {
		char* p = filename;
		char* suffix = strrchr(filename, '.');
	    
		if (suffix != NULL && strcasecmp(suffix, ".mp3") == 0)
			*suffix = '\0';
		while (*p != '\0') {
			if (*p == '_')
				*p = ' ';
			p++;
		}
	}				
	
	if (nomadII.both_present)
		bankmark = file->bank==EXTERNAL? "*": " ";
	else
		bankmark = "";

	if (file->file_size < 104857) {
		size  = file->file_size/KBF;
		units = "K";
	} else {
		size  = file->file_size/MBF;
		units = "M";
	}
							
	snprintf(buff, 5, "%d.", i+1);
	fprintf(stdout,format,bankmark,buff, filename, size, units);	
}

int list(int argc, char *argv[])
{
	int i;
		
	if (!interactive || argc == 1) {
		verb("Flash Memory Contents:\n");

		for (i = 0; i<nomadII.file_count; i++)
			if (memory_bank == BOTH || nomadII.files[i].bank == memory_bank)
				list_print(i);

		if (verbose || interactive) {
			int save_interactive = interactive;
		
			if (nomadII.both_present && memory_bank != INTERNAL)
				fprintf(stdout,"* = external memory\n");

			interactive = 0;
			count();
			interactive = save_interactive;

			if (memory() != 0)
				return -1;
		}
		return 0;
	} else if (interactive && argc == 2) {
		i = atoi(argv[1]);
		if (i < 1 || i > nomadII.file_count) {
			fprintf(stderr,"Error: no such file: %d\n", i);
			return -1;
		}
		i--; /* adjust from 1.. to 0.. space */
		list_print(i);
		return 0;
	}
	
	fprintf(stderr,"?Invalid command\n");
	return -1;	
}

int format()
{
	if (memory_bank == BOTH && nomadII.both_present) {
		fprintf(stderr,"Must specify internal or external memory (both present).\n");
		return -1;
	}

	if (!force) {
		fprintf(stdout,"Are you sure? ");

		if (!yesno()) {
			verb("Flash memory format aborted.\n");
			return 0;
		}
	}

	verb("Please wait...\n");

	if (nomadII_format_flash(&nomadII, memory_bank) != 0) {
		fprintf(stderr,"Error: Could not format flash memory!\n");
		return -1;
	}
	
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());

	verb("Flash memory formatted.\n");

	if (bell) putchar('\007');

	return 0;
}

int do_delete(int i)
{
	if (i < 1 || i > nomadII.file_count) {
		fprintf(stderr,"Error: no such file: %d\n", i);
		return -1;
	}

	i--; /* adjust from 1.. to 0.. space */

	if (verbose) fprintf(stdout,"Deleting file %s\n",nomadII.files[i].long_file_name);

	if (!force) {
		fprintf(stdout,"Deleting %d: are you sure? ", i+1);

		if (!yesno()) {
			verb("Delete file aborted.\n");
			return 0;
		}		
	}

	verb("Please wait...\n");

	if (nomadII_delete(&nomadII, i) != 0) {
		fprintf(stderr,"Error: Could not delete file: %s\n", nomadII_get_error());
		return -1;
	}

	verb("File deleted OK.\n");

	if (bell) putchar('\007');

	return 0;
}


int do_delete_all()
{
	int  i;
	char *where = "";

	if (nomadII.both_present) {
		switch(memory_bank) {
			case BOTH: 
				where = " internal and external"; 
				break;
			case INTERNAL:
				where = " internal";
				break;
			case EXTERNAL:
				where = " external";
				break;
		}
	}

	if (verbose) fprintf(stdout,"Deleting all%s files!\n", where);

	if (!force) {
		fprintf(stdout,"Are you sure you want to delete all%s files? ", where);

		if (!yesno()) {
			verb("Delete all files aborted.\n");
			return 0;
		}		
	}

	verb("Please wait...\n");

	for (i = 0; i < nomadII.file_count; i++)
		if (memory_bank == BOTH || nomadII.files[i].bank == memory_bank)
			if (nomadII_delete(&nomadII, i) != 0) {
				fprintf(stderr,"Error: deleting a file: %s\n", nomadII_get_error());
				return -1;
			}

	verb("All files deleted OK.\n");

	return 0;
}
		

int delete(int argc, char *argv[])
{
	int i;

	if (argc == 1) {
		fprintf(stderr,"Usage: delete all|<n> ...\n");
		return -1;
	}

	if (argc == 2 && strcmp(argv[1], "all") == 0) {
		if (do_delete_all() == -1)
			return -1;
	} else {
		for (i = 1; i < argc; i++)
			if (do_delete(atoi(argv[i])))
				return -1;
	}
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
		
	return 0;
}

int show_presets()
{
	unsigned int i;

	if (!nomadII.has_radio) {
		fprintf(stderr,"This model has no radio!\n");
		return -1;
	}

	for (i=0; i<=31; i++)
		fprintf(stdout,"Preset #%-2d  %3d.%02d\n",i+1,nomadII.radios[i].freq,
			nomadII.radios[i].subfreq);
	return 0;
}

int get_presets()
{
	if (!nomadII.has_radio) {
		fprintf(stderr,"This model has no radio!\n");
		return -1;
	}

	if (interactive)
		if (nomadII_read_presets(&nomadII) != 0) {
			fprintf(stderr,"Error: Could not get radio presets: %s\n", nomadII_get_error());
			return -1;
		}
	
	verb("Radio presets loaded.\n");
	return 0;
}

int set_preset(unsigned int preset, unsigned int freq, unsigned int subfreq)
{
	unsigned int kHzDiv10, lastDigit;

	if (!nomadII.has_radio) {
		fprintf(stderr,"This model has no radio!\n");
		return -1;
	}

	preset--; /* adjust from 1.. to 0.. space */

	kHzDiv10 = freq*100 + subfreq;

	if (kHzDiv10 < 8750 || kHzDiv10 > 10800) {
		fprintf(stderr,"Error: supported frequency range is 87.5 - 108.0MHz: %d.%02dMHz\n", 
			freq, subfreq);
		return -1;
	}

	lastDigit = kHzDiv10 % 10;
	if (lastDigit != 0) {
		if (lastDigit <= 5)
			subfreq -= lastDigit;
		else
			subfreq += (10 - lastDigit);
			
		if (subfreq == 100) {/* rounded up */
			subfreq = 0;
			freq++;
		}
		fprintf(stderr,"Warning: frequency can only set in steps of 0.10MHz - using %d.%02dMHz\n", 
			freq, subfreq);
	}


	if (preset >= 0 && preset < RADIO_PRESETS) {
		nomadII.radios[preset].freq = freq;
		nomadII.radios[preset].subfreq = subfreq;
	} else {
		fprintf(stderr,"Error: Invalid preset number!\n");
		return -1;
	}

	return 0;
}

int set_presets()
{	
	if (!nomadII.has_radio) {
		fprintf(stderr,"This model has no radio!\n");
		return -1;
	}

	if (nomadII_write_presets(&nomadII) != 0) {
		fprintf(stderr,"Error: Could not set radio presets: %s\n", nomadII_get_error());
		return -1;
	}
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
	
	verb("Radio presets written.\n");
	return 0;
}

int presets(int argc,char *argv[])
{
	if (!nomadII.has_radio) {
		fprintf(stderr,"This model has no radio!\n");
		return -1;
	}

	if (interactive) {

		switch (argc) {
		
		case 1:	
			show_presets();
			break;
		
		case 2:		  
			if (!strcmp(argv[1],"all")) {			
				show_presets();
				return 0;
			}
			else if (!strcmp(argv[1],"read")) {
				return get_presets();
			}
			else if (!strcmp(argv[1],"write")) {
				return set_presets();
			}
			else fprintf(stderr,"?Invalid command\n");
			break;

		case 4:
			if (!strcmp(argv[1],"set")) {
				if (strchr(argv[3], '.') != NULL) {
					fprintf(stderr, "Error: frequency must be set in the form 9390 (for 93.90MHz)\n");
					return -1;
				}
				if (strcmp(argv[2], "all") == 0) {
					int i;
					if (!force) {
						fprintf(stdout,"Are you sure you want to set all frequencies? ");

						if (!yesno()) {
							verb("Set all frequencies aborted.\n");
							return 0;
						}		
					}
					for (i=1; i <= 32; i++)
						if (set_preset(i,atoi(argv[3])/100,atoi(argv[3])%100) != 0)
							return -1;
					return 0;
				} else {
					return set_preset(atoi(argv[2]),atoi(argv[3])/100,atoi(argv[3])%100);
				}
			}
			else fprintf(stderr,"?Invalid command\n");
			break;

		default:
			fprintf(stderr,"?Invalid command\n");
		
		}

	} else {
		/* command line option */
		if (get_presets() != 0) return -1;
		show_presets();
		return 0;

	}
	return -1;
}

int do_download(char *file_name) 
{
	int bank_used;

	if (!nomadII.present[INTERNAL] && !nomadII.present[EXTERNAL]) {
		fprintf(stderr,"Cannot send to Nomad II with no memory present.\n");
		return -1;
	}

	if (verbose) fprintf(stdout,"Sending file %s...\n",file_name);
	if (nomadII_download_file(&nomadII, memory_bank, file_name, &bank_used) != 0) {
		fprintf(stderr,"Error: Could not send file %s: %s\n",file_name, nomadII_get_error());
		return -1;
	}
	
	/* since didn't fit in internal, stay on external for rest of downloads */
	if (memory_bank == BOTH && bank_used == EXTERNAL)
		memory_bank = EXTERNAL;
	
	if (verbose) fprintf(stdout,"File %s sent succesfully\n",file_name);
	
	return 0;
}

int download(int argc, char *argv[])
{
	int i, saved_memory_bank = memory_bank; /* can change in do_download */
	
	if (argc < 2) {
		fprintf(stderr,"Usage: %s <name> ...\n", argv[0]);
		return -1;
	}
       
	progress_handler(NOMADII_PROGRESS_RESET, argc - 1, NULL);
	for (i = 1; i < argc; i++)
		if (do_download(argv[i]) == -1) {
			memory_bank = saved_memory_bank;
			return -1;
		}
	memory_bank = saved_memory_bank;
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
	if (bell) putchar('\007');
	return 0;
}

int get_next_playlist_item(FILE* playlist_file, char* file_name)
{
	int len;

	while (feof(playlist_file) == 0) {
		fgets(file_name,PATH_MAX,playlist_file);
		len = strlen(file_name);
		if (file_name[len-1] == '\n') {
			if (file_name[len-2] == '\r') /* crlf's present */
				file_name[strlen(file_name)-2] = '\0';
			else
				file_name[strlen(file_name)-1] = '\0';
		}
		if (feof(playlist_file) != 0)
			break;
			
		if (file_name[0]=='#' || file_name[0]=='\n' ||
				file_name[0]=='\0' || file_name[0] ==' ')
			continue;
		if (access(file_name, R_OK) == -1) {
			fprintf(stderr, "playlist file %s: %s\n", 
				file_name, strerror(errno));
			return -1;
		}
		return 0;
	}
	return -2;
}


int do_playlist(char *playlist_file_name, int offset, int number)
{
	FILE *playlist_file;
	char file_name[PATH_MAX];
	int result, total, count, song;
	int bank_used, bank = memory_bank;

	if (!nomadII.present[INTERNAL] && !nomadII.present[EXTERNAL]) {
		fprintf(stderr,"Cannot send playlist files with no memory present.\n");
		return -1;
	}

        /* scan playlist and download files one by one */
	if ((playlist_file = fopen(playlist_file_name,"r")) == NULL) {
		fprintf(stderr,"Error: could not open playlist file %s: %s\n", 
			playlist_file_name, strerror(errno));
		return -1;
	}

	count = 0;

	/* check playlist for files first */
	while (feof(playlist_file) == 0) {
		result = get_next_playlist_item(playlist_file, file_name);
		if (result == -2 && count == 0) {
			fprintf(stderr,"Error: playlist file %s does not contain any files\n", 
				playlist_file_name);
			return -1;
		} else if (result == -1)
			return -1;
		else if (result == 0)
			count++;
	}
	
	song = 1;
	rewind(playlist_file);
	
	total = count - offset + 1;
	if (number >= 0 && total > number)
		total = number;

	if (number > total)
		fprintf(stderr, "Warning: less than %d files in playlist.\n", number);
	
	progress_handler(NOMADII_PROGRESS_RESET, total, NULL);

	while (count-- && feof(playlist_file) == 0) {
		if (get_next_playlist_item(playlist_file, file_name) < 0)
			return -1;
		
		/* Offset playlist support */
		if (song++ < offset) continue;

		if (prompt) {
			fprintf(stdout,"Send file %s? ",file_name);

			if (!yesno()) {
				if (verbose) 
					fprintf(stdout,"Send file %s aborted.\n",file_name);
				continue;
			}
		}

		/* Download */
		if (verbose) fprintf(stdout,"Sending file %s...\n",file_name);
		if (nomadII_download_file(&nomadII,bank,file_name,&bank_used) != 0) {
			fprintf(stderr,"Error: Could not send file %s: %s\n",file_name,nomadII_get_error());
			return -1;
		}
		/* since didn't fit in internal, stay on external for rest of playlist */
		if (bank == BOTH && bank_used == EXTERNAL)
			bank = EXTERNAL;
		if (verbose) fprintf(stdout,"File %s sent succesfully\n",file_name);
		if (nomadII_sync(&nomadII) != 0)
			fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
		if (--number == 0)
			break;
	}
	
	fclose(playlist_file);

	if (verbose) fprintf(stdout,"Playlist %s contents sent succesfully\n",playlist_file_name);

	return 0;
}

int playlist(int argc, char *argv[])
{
	int offset, number, result;

	if (argc < 2 || argc > 4) {
		fprintf(stderr,"Usage: playlist file_name [offset [number]]\n");
		return -1;
	}

	offset = argc > 2  ? atoi(argv[2]) : 0;
	number = argc == 4 ? atoi(argv[3]) : -1;

	result = do_playlist(argv[1],offset,number);
	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
	if (bell) putchar('\007');
	return result;
}

int do_upload(int i) 
{
	if (i < 1 || i > nomadII.file_count) {
		fprintf(stderr,"Error: no such file: %d\n", i);
		return -1;
	}

	i--; /* adjust from 1.. to 0.. space */

	if (verbose) fprintf(stdout,"Getting file %s...\n",nomadII.files[i].long_file_name);

	if (nomadII_upload_file(&nomadII,i) != 0) {
		fprintf(stderr,"Error: Could not get file %s from Nomad II: %s\n",
		    nomadII.files[i].long_file_name, nomadII_get_error());
		return -1;
	}

	if (verbose) fprintf(stdout,"File %s copied from Nomad II.\n",nomadII.files[i].long_file_name);

	return 0;
}

int do_upload_all()
{
	int  i;
	char *where = "";

	if (nomadII.both_present) {
		switch(memory_bank) {
			case BOTH: 
				where = " internal and external"; 
				break;
			case INTERNAL:
				where = " internal";
				break;
			case EXTERNAL:
				where = " external";
				break;
		}
	}

	if (verbose) fprintf(stdout,"Copying all%s files from Nomad II.\n", where);

	progress_handler(NOMADII_PROGRESS_RESET, nomadII.file_count, NULL);
	for (i = 0; i < nomadII.file_count; i++)
		if (memory_bank == BOTH || nomadII.files[i].bank == memory_bank)
			if (nomadII_upload_file(&nomadII,i) != 0) {
				fprintf(stderr,"Error: copying file from Nomad II: %s\n", nomadII_get_error());
				return -1;
			}
	if (bell) putchar('\007');
	return 0;
}
		

int upload(int argc, char *argv[])
{
	int i;

	if (argc < 2) {
		fprintf(stderr,"Usage: %s all|<n> ...\n", argv[0]);
		return -1;
	}
	
	if (argc == 2 && strcmp(argv[1], "all") == 0) {
		if (do_upload_all() == -1)
			return -1;
	} else {
		progress_handler(NOMADII_PROGRESS_RESET, argc - 1, NULL);
		for (i = 1; i < argc; i++)
			if (do_upload(atoi(argv[i])) == -1)
				return -1;
	}

	if (nomadII_sync(&nomadII) != 0)
		fprintf(stderr,"Warning: Unable to get in sync with device: %s\n", nomadII_get_error());
	if (bell) putchar('\007');
	return 0;
}


int lcd(int argc, char *argv[])
{
	char buf[PATH_MAX];
	const char *dir = NULL;
	
	if (argc == 1 || strcmp(argv[0], "lpwd") == 0) {
		if (!getcwd(buf, sizeof(buf))) {
			if (errno==ERANGE) 
				fprintf(stdout,"<too long>\n");
			else
				fprintf(stdout,"Error getting local working directory: %s\n", strerror(errno));
		} else
			fprintf(stdout,"Local directory: %s\n", buf);
		return 0;
	}
	
	if (argc != 2) {
		fprintf(stderr,"Usage: lcd local-directory\n");
		return -1;
	}
	else dir = argv[1];
	
	if (!dir) return -1;
	
	if (chdir(dir) < 0) {
		fprintf(stderr, "Error: %s: %s\n", dir, strerror(errno));
		return -1;
	}
	if (!getcwd(buf, sizeof(buf))) {
		if (errno==ERANGE) strcpy(buf, "<too long>");
		else snprintf(buf, PATH_MAX, "Error getting local working directory: %s\n", strerror(errno));
	}
	if (verbose) fprintf(stdout,"Local directory now: %s\n", buf);
	return 0;
}

int opt_autorename() 
{
	autorename = !autorename;
	fprintf(stdout,"Autorename mode %s.\n", onoff(autorename));
	return 0;
}

int opt_verbose() 
{
	verbose = !verbose;
	fprintf(stdout,"Verbose mode %s.\n", onoff(verbose));
	return 0;
}

int opt_bell()
{
	bell = !bell;
        fprintf(stdout,"Bell mode %s.\n", onoff(bell));
	return 0;
}

int opt_hash()
{
	hash = !hash;
        fprintf(stdout,"Hash mark printing %s.\n", onoff(hash));
	return 0;
}

int opt_prompt()
{
	prompt = !prompt;
	fprintf(stdout,"Prompt mode %s.\n", onoff(prompt));
	return 0;
}

int opt_force()
{
	force = !force;
	fprintf(stdout,"Force mode %s.\n", onoff(force));
	if (force) verb("Be careful!\n");
	return 0;
}

int opt_internal()
{
	if (!nomadII.present[INTERNAL]) {
		fprintf(stdout,"No internal flash memory available.\n");
	} else {
	    memory_bank = INTERNAL;
	    fprintf(stdout,"Using internal flash memory.\n");
	}
	return 0;
}

int opt_external()
{
	if (!nomadII.present[EXTERNAL]) {
		fprintf(stdout,"No external flash memory available.\n");
	} else {
		memory_bank = EXTERNAL;
		fprintf(stdout,"Using external flash memory.\n");
	}
	return 0;
}

int opt_both()
{
	if (nomadII.both_present) {
		memory_bank = BOTH;
		fprintf(stdout,"Using internal and external flash memory.\n");
	} else {
		fprintf(stdout,"Error: both internal and external memory must be present.\n");
	}
	return 0;
}

#ifdef PACKET_DUMP
int opt_debug()
{
	nomadIIdebug++;
	if (nomadIIdebug > 2)
		nomadIIdebug = 0;
	debug = (nomadIIdebug > 0);
	fprintf(stdout,"Debug mode (packet dump) level %d\n", nomadIIdebug);
	return 0;
}
#endif

int yesno ()
{
  char buf[128];
  int len = 0;
  int c;

  while ((c = getchar ()) != EOF && c != '\n')
    if ((len > 0 && len < 127) || (len == 0 && !isspace (c)))
      buf[len++] = c;
  buf[len] = '\0';

  return (buf[0]=='Y' || buf[0]== 'y');
}
