/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#ifndef _NOMADII_H_
#define _NOMADII_H_

#include <sys/param.h>
#include <time.h>

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

typedef struct nomadII_device* NomadIIHandle;

extern int nomadIIdebug;

/* NOMADII Memory Banks */
#define MEMORY_BANKS    2
#define INTERNAL 	0
#define EXTERNAL	1
#define BOTH		2

/* NOMADII File Types */
#define FILE_ANY     0
#define FILE_MP3     1
#define FILE_NVF     2
#define FILE_WMA     4
#define FILE_AUX     5
#define FILE_UNKNOWN 6

/* Progress Message Type */
#define NOMADII_PROGRESS_NEW_FILE  0
#define NOMADII_PROGRESS_UPDATE    1 
#define NOMADII_PROGRESS_DUPCRC    2 /* duplicated CRC found at 512 block N */
#define NOMADII_PROGRESS_RESET     3 /* pass number of items  */

/* NOMADII Radio Presets */
#define RADIO_PRESETS 32

/* NOMADII Max Number of Files */
#define MAX_FILES	    1024
#define MAX_OWNER_LENGTH    0x09

/***************************************************
 * NOMADII structures
 * - in time these will become opaque
 ***************************************************/
#define STRING_SIZE 0x100

typedef unsigned char  BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
typedef char string[256];
typedef int boolean;

typedef struct {
	char   long_file_name[PATH_MAX];
	char   file_name[9];
	char   file_ext[4];
	char   type;
	int    bank;
	time_t mtime;
	DWORD  file_size;
} nomadII_file;

typedef struct {
	short freq,subfreq;
} preset_entry; 

typedef struct nomadII_device {
	int bus, dev, vendor_id, prod_id;
	char dst_dev[PATH_MAX+1];
	int fd;
	int nomadII_isopen, found;
	int mem_total[MEMORY_BANKS];
	int mem_free[MEMORY_BANKS];
	int bank_count[MEMORY_BANKS];
	int file_count;
	int present[MEMORY_BANKS];
	int both_present;
	nomadII_file files[MAX_FILES];
	string model_info, owner_info, fw_info;
	int has_radio;
	preset_entry radios[RADIO_PRESETS];
	struct tm dclock;
} nomadII_device;

/***************************************************/

typedef void nomadII_progress_handler_cb(int message, int argc, void* data);

/* NOMADII Function prototypes */
char* nomadII_get_library_version();
char* nomadII_get_error();
void  nomadII_set_progress_handler(nomadII_progress_handler_cb*, void* data, int ticks);

int nomadII_probe(NomadIIHandle);
int nomadII_open(NomadIIHandle);
int nomadII_close(NomadIIHandle);
int nomadII_sync(NomadIIHandle);
int nomadII_prepare(NomadIIHandle);
int nomadII_model(NomadIIHandle);
int nomadII_get_clock(NomadIIHandle);
int nomadII_set_clock(NomadIIHandle);
int nomadII_get_owner(NomadIIHandle);
int nomadII_set_owner(NomadIIHandle);
int nomadII_memory(NomadIIHandle);
int nomadII_count_files(NomadIIHandle);
int nomadII_get_files(NomadIIHandle);
int nomadII_format_flash(NomadIIHandle, int);
int nomadII_read_presets(NomadIIHandle);
int nomadII_write_presets(NomadIIHandle);
int nomadII_delete(NomadIIHandle, int number);
int nomadII_download_file(NomadIIHandle, int bank, char *file_name, int *bank_used);
int nomadII_upload_file(NomadIIHandle, int number);
int nomadII_longname_exists(NomadIIHandle, char* long_name, int bank);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* _NOMADII_H_ */
