/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#ifndef _NOMADII_PRIV_H_
#define _NOMADII_PRIV_H_

#include "nomadII.h"

#define NOMADII_LIB_VERSION "0.8"

#ifndef WITH_USBDEVFS
/***************************************************
 * low level driver
 */
#define NOMADII_SEND_COMMAND			0x1
#define NOMADII_RECV_COMMAND			0x2
#define NOMADII_ASYNC_COMMAND                   0x3
#define NOMADII_ASYNC_COMPLETE                  0x4
#define NOMADII_RESET_PIPES                     0x5

struct NomadII_cmd {
	short length;
	int request;
	int requesttype;
	int value;
	int index;
	void *data;
	int timeout;
};
#endif
/***************************************************/
 
#define MAX_ERROR_LENGTH	    1024
extern char nomadIIErrorBuff[];
 
extern nomadII_progress_handler_cb*	    nomadII_progress_handler;
extern void*				    nomadII_progress_data;
extern int				    nomadII_progress_ticks;
 
/* NOMAD II Command Result codes */
#define RESULT_ILLEGAL_FILENAME     0xE6 /* 230 */
#define RESULT_ILLEGAL_FREQUENCY    0xD0 /* 208 */
#define RESULT_ILLEGAL_FREQ_DECIMAL 0xD1 /* 209 */
#define RESULT_NO_SPACE_LEFT	    0xE5 /* 229 */
#define RESULT_FILE_EXISTS	    0xA0 /* 160 */
/* this is returned if zero bytes is written */
#define RESULT_ILLEGAL_ARGUMENT	    0xA3 /* 163 */ 


/* NOMADII Low level byte offsets */
#define OFFSET_MODEL    	    0x30
#define OFFSET_OWNER    	    0x00
#define OFFSET_FILE_COUNT	    0x01
#define OFFSET_LONG_FILE_NAME	    0x001
#define OFFSET_FILE_NAME	    0x201
#define OFFSET_FILE_EXT 	    0x209
#define OFFSET_FILE_SIZE	    0x20C
#define OFFSET_FILE_TYPE	    0x210
#define OFFSET_FILE_TIME	    0x211
#define OFFSET_FILE_DATE	    0x213
#define OFFSET_MEM_INTERNAL_TOTAL   0x01
#define OFFSET_MEM_INTERNAL_FREE    0x05
#define OFFSET_MEM_EXTERNAL_TOTAL   0x09
#define OFFSET_MEM_EXTERNAL_FREE    0x0D

#define OFFSET_FW_MAJOR		    1
#define OFFSET_FW_MINOR_MG	    6
#define OFFSET_FW_SUBVER	    5

/* NOMADII Block Size in transfers */
#define NOMAD_BLOCK_SIZE            0x0200
#define DOWNLOAD_BLOCK_SIZE         0x1000

/* max bulk transfer is a 4096 byte page, and
   Nomad requires is to transfer an extra 0x40
   bytes for some reason, so to align it with
   512 byte blocks (which Nomad sometimes
   repeats) get 7 blocks */
#define UPLOAD_BLOCK_SIZE           0x0E00
#define UPLOAD_BLOCK_HEADER         0x0040

/* NOMADII Commands */
#define COMMAND_FILE_COUNT          0x01
#define COMMAND_FILE_INFO           0x02
#define COMMAND_SELECT_FILE         0x03
#define COMMAND_END_TRANSFER        0x04
#define COMMAND_SEND_BLOCK          0x05
#define COMMAND_RCV_BLOCK           0x06
#define COMMAND_DELETE_FILE         0x08
#define COMMAND_MEMORY              0x0D
#define COMMAND_MODEL               0x0E
#define COMMAND_FORMAT              0x0F
#define COMMAND_PREPARE2            0x10
#define COMMAND_GET_CLOCK           0x11
#define COMMAND_SET_CLOCK           0x12
#define COMMAND_PREPARE             0x1A
#define COMMAND_GET_OWNER           0x1C
#define COMMAND_SET_OWNER           0x1D
#define COMMAND_GET_RADIO_PRESETS   0x1E
#define COMMAND_SET_RADIO_PRESETS   0x1F
#define COMMAND_BEGIN_TRANSFER      0x25
#define COMMAND_GET_RESULT          0xF0

#define NOMADII_TIMEOUT_SMALL            150
#define NOMADII_TIMEOUT_MEDIUM           2500
#define NOMADII_TIMEOUT_LARGE            15000

#define NOMADII_BULK_IN                  0x02
#define NOMADII_BULK_OUT                 0x01

#define BULK_TIMEOUT                     10000
#define BULK_MAX_RETRIES                 3
#define BULK_INTERVAL                    5*HZ
#define UPLOAD_MAX_RETRIES               5

/*
 *  Low level commands in nomadII_low_level.c
 */

#define CHECKFD()   if (fd < 0) { errno = EBADF; return -1; }
#define CHECKOPEN(x) nomadII_reset_error(); \
    if (!x->nomadII_isopen) { errno = EBADF; return -1; }

int nomadII_read_command(int fd, int req, int val, int idx, int len, BYTE *buffer);
int nomadII_write_command(int fd, int req, int val, int idx, int len, BYTE *buffer);
int nomadII_read(int fd, BYTE *buffer, int count);
int nomadII_write(int fd, BYTE *buffer, int count);
int nomadII_send_command(int fd, int req, int val, int idx);
int nomadII_send_command_nowait(int fd, int req, int val, int idx);
int nomadII_command_wait(int fd);
int nomadII_reset_pipes(int fd);
int nomadII_ctl_msg(int fd, int dir, int req, int val, int idx, int len, BYTE *d);
#ifdef WITH_USBDEVFS
int nomadII_bulk_msg(int fd, int direction, int len, BYTE *data);
#endif

/*
 *  Util commands in nomadII_util.c
 */

#define nomadII_reset_error() nomadIIErrorBuff[0] = '\0'
void   nomadII_set_error(char* format, ...);
int    nomadII_build_dos_name(NomadIIHandle, char* out, char* in, int bank, char* suffix);
time_t nomadII_date_dos2unix(WORD dosdate, WORD dostime);
void   nomadII_date_unix2dos(int unix_date, WORD *dosdate, WORD *dostime);
void   nomadII_set_file_name(char *src, char *dst); 
void   nomadII_set_file_ext(char *src, char *dst);
void   nomadII_set_long_file_name(char *src, char *dst);
unsigned long nomadII_adler32(const BYTE* buf, int len);
#ifdef PACKET_DUMP
int    nomadII_hex_dump(char *data, long size);
#endif

#endif /* _NOMADII_PRIV_H_ */
