/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#include "nomadIIpriv.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#define UPLOAD_INTERVAL  500

static int nomadII_rcv_block(int fd, BYTE *buffer,unsigned int partial_size, unsigned int block_size)
{
	/* CONTROL 0x06 --> DOWNSTREAM (OUT) (data sizes) (length 0x08) */
	/* CONTROL <-- UPSTREAM (IN)                                    */
	/* BULK --> DOWNSTREAM (IN) (length block_size+0x40)            */
	/* BULK <-- UPSTREAM   (IN) (data block)                        */

	BYTE command[0x08];

	memcpy(&command[0x00],&partial_size,sizeof(int));
	memcpy(&command[0x04],&block_size,sizeof(int));

	if (nomadII_write_command(fd, COMMAND_RCV_BLOCK,0x00,0x00,0x08,command) != 0)
		return -1;

	if (nomadII_read(fd, buffer,block_size+UPLOAD_BLOCK_HEADER) != 0)
		return -1;

	return 0;
}

int nomadII_upload_file(NomadIIHandle nomadII, int number) 
{
  	int dst_file,i,retry,repeated,errno,count;
	unsigned int total_size, block_size, current_size, partial_size, file_size;
	float accum;
	int progress, accum_step;
	unsigned long last_crc, current_crc, crc_save;
	BYTE buffer[UPLOAD_BLOCK_SIZE+UPLOAD_BLOCK_HEADER];
	BYTE* buffer_pos;
	BYTE status[2];
	BYTE info[]    = { 0x00, 0x00, 0x00,   
			   0x00, 0x00, 0x00,   
			   0x00, 0x00, 0x00,
			   0x00, 0x00, 0x00  };
	
	char *file_name      = nomadII->files[number].file_name;
	char *file_ext       = nomadII->files[number].file_ext;
	char *long_file_name = nomadII->files[number].long_file_name;
	int   bank           = nomadII->files[number].bank;

	BYTE command[] = { 0x00, 0x00, 0x00,
			   0x00, 0x00, 0x00,
			   0x00, 0x00, 0x00,
			   0x00, 0x00 };

	for (i = 0; i <= 7; i++)
		command[i] = *(file_name+i);
	for (i = 0; i <= 2; i++)
		command[i+8] = *(file_ext+i);

	if ((dst_file = open(long_file_name,O_RDWR | O_CREAT,S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0) {
		nomadII_set_error("open for writing - %s", strerror(errno));
		return -1;
	}

	/* CONTROL 0x25 --> DOWNSTREAM (IN)                */
	/* CONTROL <-- UPSTREAM (IN) (length 0x02) (01 00) */
	if (nomadII_read_command(nomadII->fd,COMMAND_BEGIN_TRANSFER,0x00,0x00,0x02,status) != 0) {
		close(dst_file);
		return -1;
	}

	if (status[0] != 0x00 || status[1] != 0x01) {
		close(dst_file);
		errno = EIO;
		return -1;
	}

	/* CONTROL 0x03 val 0x01 --> DOWNSTREAM (IN)             */
	/* CONTROL <-- UPSTREAM (IN) (length 0x0C) (data size)   */
	/* BULK --> DOWNSTREAM (OUT) (length 0x17) (data name)   */
	/* BULK <-- UPSTREAM   (OUT)                             */
	/* CONTROL 0xF0 --> DOWNSTREAM (IN)                      */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00)          */
	if (nomadII_write_command(nomadII->fd,COMMAND_SELECT_FILE,0x0100|bank,0x00,0x0C,info) != 0) {
		close(dst_file);
		return -1;
	}

	if (nomadII_write(nomadII->fd,command,sizeof(command)) != 0) {
		close(dst_file);
		return -1;
	}

	/* CONTROL 0xF0 --> DOWNSTREAM (IN)             */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00) */
 	if (nomadII_send_command(nomadII->fd,COMMAND_GET_RESULT,0x00,0x00) != 0) {
		close(dst_file);
		return -1;
	}

	/*
	 * Block transfers
	 */
	/* progress - nomadII_progress_ticks ticks */
	accum    = 0.01; /* prime to cope with rounding error */
	progress = 0;

	total_size = file_size = nomadII->files[number].file_size;

	partial_size = 0;
	last_crc     = 0;

	if (nomadII_progress_handler)
		nomadII_progress_handler(NOMADII_PROGRESS_NEW_FILE, 0, nomadII_progress_data);

	retry = 0;
	
	while(total_size) {
		DWORD bytes_read;

		repeated = 0;
		current_size = block_size = (total_size <= UPLOAD_BLOCK_SIZE) ? total_size : UPLOAD_BLOCK_SIZE;
		buffer_pos   = &buffer[UPLOAD_BLOCK_HEADER];

		if (nomadII_rcv_block(nomadII->fd,buffer,partial_size,block_size) != 0) {
			close(dst_file);
			return -1;
		}

		memcpy(&bytes_read ,&buffer[0x01],sizeof(DWORD));
		if (bytes_read != block_size) {
			if (retry++ > UPLOAD_MAX_RETRIES) {
				nomadII_set_error("error: failed to correct for short read at block %d (%d retries)",
				    partial_size/NOMAD_BLOCK_SIZE, retry);
				close(dst_file);
				errno = EIO;
				return -1;
			}
			continue;
		}

		crc_save = last_crc; /* if we reread 7 block section, need to restore last_crc */
		for(i = 0; i < 7 && current_size > 0; i++) {
			int crc_len = (current_size < NOMAD_BLOCK_SIZE) ? current_size : NOMAD_BLOCK_SIZE;
			current_crc = nomadII_adler32(buffer_pos, crc_len);
			
			/* very low probability of false positives unless blank data */
			if (current_crc == last_crc && current_crc != 0) {
				repeated = (partial_size+i*NOMAD_BLOCK_SIZE)/NOMAD_BLOCK_SIZE;
				nomadII_progress_handler(NOMADII_PROGRESS_DUPCRC, 
					repeated, nomadII_progress_data);
				last_crc = crc_save;
				break;
			}
			last_crc = current_crc;
			buffer_pos   += crc_len;
			current_size -= crc_len;
		}

		if (repeated) {
			if (retry++ > UPLOAD_MAX_RETRIES) {
				nomadII_set_error("error: failed to correct for duplicated block %d (%d retries)",
				    repeated, retry);
				close(dst_file);
				errno = EIO;
				return -1;
			}
			continue;
		}

		retry = 0; /* reset on success */

		/* Block OK */
		if ((count = write(dst_file,&buffer[UPLOAD_BLOCK_HEADER],block_size)) != block_size) {
			close(dst_file);
			return -1;
		}

		if (nomadII_progress_handler) {
			/* proportion completed out of nomadII_progress_ticks*/
			accum     += (float)(count*nomadII_progress_ticks)/(float)file_size;
			accum_step = (int)accum;
		
			while (accum_step > progress)
				nomadII_progress_handler(NOMADII_PROGRESS_UPDATE, progress++, nomadII_progress_data);
		}

		total_size -= block_size;
		partial_size += block_size;
		usleep(UPLOAD_INTERVAL);
	}
	
	if (nomadII_progress_handler && progress <= nomadII_progress_ticks)
		nomadII_progress_handler(NOMADII_PROGRESS_UPDATE, nomadII_progress_ticks, nomadII_progress_data);

	/* CONTROL 0x04 --> DOWNSTREAM (IN)             */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00) */
	if (nomadII_send_command(nomadII->fd,COMMAND_END_TRANSFER,0x00,0x00) != 0) {
		close(dst_file);
		return -1;
	}

	if (nomadII_memory(nomadII) != 0) {
		close(dst_file);
		return -1;
	}

	close(dst_file);

	return 0;
}
