.TH NOMADII 1 "MARCH 2002" Linux "User Manuals"
.SH NAME 
.B nomadii
- a tool that controls NOMAD II MP3 players
.SH SYNOPSIS
.B nomadii [-v, --verbose] [-b, --bell] [-H, --hash]
.B [-i, --internal] [-x, --external] [-t, --prompt] [-F, --force]  
.B [-o, --offset <n>] [-n, --number <n>] [-r, --replace] 
.B [--freq <n>] [--debug] [command]

.B nomadii -V, --version

.B nomadii -h, --help
.SH DESCRIPTION
.B nomadii
is a command line tool for controlling the Creative Nomad II, IIc and II MG Portable MP3 players.
It has two modes of operation: command line arguments and an ftp-like interactive console. This
man page is up to date as of Version 0.8 of 
.BR nomadii .

.SH COMMAND LINE ARGUMENTS

.I COMMANDS

Only a single command can be specified, and it must come at the
end of the argments.

.B "-P, --probe"
.RS 
probe USB bus for Nomad II devices
.RE

.B "-M, --model"
.RS
display Nomad II model and firmware information
.RE

.B "-m, --memory"
.RS
display Nomad II memory usage:
Total/Used/Free in MB
.RE

.B "-c, --count"
.RS
display number of files in flash memory
.RE

.B "-l, --list"
.RS
display the list of files in flash memory 
.RE

.B "-f, --format"
.RS
formats flash memory; see
.I -i
and
.IR -x
if both internal and external memory is installed
.RE

.B "-D, --delete all|<n> ..."
.RS
delete files
with the specified numbers from flash memory or all files with the keyword
.IR all ;
files are numbered beginning with 1
.RE

.B "-s, --send, --put, -d, --download <file> ..."
.RS
send the
specified files from computer to Nomad II flash memory;
use with
.I --hash
to disable copy progress. MP3 files must have an .mp3 suffix
and NVF files must have an .nvf
suffix to be recognized by the player.
.RE

.B "-g, --get, -u, --upload all|<n> ..."
.RS
upload files
with the specified numbers from Nomad II flash memory to computer
or all files with the keyword
.IR all ;
use with
.I --hash
to disable copy progress.
.RE

.B "-p, --playlist <name>"
.RS
download all files in playlist file
.I name
from computer to Nomad II flash memory. Use with
.I --hash
to disable download progress; 
playlists must be in xmms or winamp like format.
See options
.IR --offset,
.I --number
and
.IR --prompt .
.RE

.B "--clock"
.RS
displays date and time from Nomad II
.RE

.B "--clock-set"
.RS
sets Nomad II date and time to now
.RE

.B "--owner"
.RS
displays Nomad II owner string
.RE

.B "--owner <name>"
.RS
sets Nomad II owner string to
.I name
.RE

.B "--presets"
.RS
display list of Nomad II FM radio presets (not supported on IIc)
.RE

.B "--presets-set <n>"
.RS
sets Nomad II FM radio preset number 
.I n
(not supported on IIc); you must use this command in conjunction with the
.I --freq
option
.RE

.I OPTIONS

.B "-a, --autorename"
.RS
turns off auto-rename function; autorename turns underscores into
spaces and strips off the .mp3 suffix when listing MP3 files.
.RE

.B "-b, --bell"
.RS
enables sound bell/beep on transfer completion;
bell is disabled by default
.RE

.B "-H, --hash"
.RS
disables transfer progress display (no longer hash marks);
hash mode is enabled by default
.RE

.B "-i, --internal"
.RS
use the internal flash memory for operations when both
internal and external memory is present. 
.I Format 
requires
.I -i 
or 
.I -x
to be specified when both internal and external memory
is present.
.IR List ,
.IR send ,
.IR get ,
.IR playlist ,
.I memory
and
.IR count
are effected by this mode.
.RE

.B "-x, --external"
.RS
use the external flash memory for operations when both
internal and external memory is present. 
.I Format 
requires
.I -i 
or 
.I -x
to be specified when both internal and external memory
is present.
.IR List ,
.IR send ,
.IR get ,
.IR playlist ,
.I memory
and
.IR count
are effected by this mode.
.RE

.B "-t, --prompt"
.RS
enable interactive mode in playlist downloads.
For each song in the playlist 
.I nomadii 
will ask for confirmation.
Prompt is disabled by default. 
.RE

.B "-F, --force"
.RS
do not ask for confirmation on delete and format commands;
force is disabled by default
.RE

.B "-o, --offset <n>"
.RS
make playlist transfer start at file number
.I n
in the playlist
.RE

.B "-n, --number <n>"
.RS
make playlist transfer only
.I n
items from the playlist
.RE

.B "-r, --replace"
.RS
this option deletes all items before sending
when used with 
.I --send
and
.I --playlist
.RE

.B "--freq <f1f2>"
.RS
specifies FM radio frequency
.I f1.f2MHz
for command
.IR --presets-set .
For FM freqency 93.90MHz you must specify
.IR 9390 .
.RE

.B "--debug"
.RS
If compiled with DEBUG enabled (see Makefile) this option increases the debug
level. Level 
.B 1
shows packet dumps for errors; level
.B 2 
shows packet dumps for all USB communication.
Debug is disabled by default.
.RE

.SH INTERACTIVE CONSOLE

.I COMMANDS

.BR quit , 
.BR exit , 
.B bye
.RS
exit program
.RE
 
.B help         
.RS
get list of commands
.RE

.B help <command>
.RS
get a short description of command syntax and function
.RE

.B status
.RS
show open/close device state and display state of options
.RE

.BR autorename ,
.BR bell , 
.BR hash , 
.BR debug , 
.BR force , 
.B  verbose
.RS
toggle the option named by the comand. See the
.I OPTIONS 
section for details on these flags. The debug option cycles the
debug level between off, 1 and 2.
.RE

.BR lcd , 
.B lpwd
.RS
print local working directory
.RE

.B lcd <local-dir>
.RS
change local working directory to 
.I local-dir
.RE

.B open         
.RS
open device - it is called automatically if 
.I nomadii 
finds a Nomad II connected at startup.
You must call it yourself if you closed the device manually during operation.
.RE

.B close        
.RS
closes the device;
it is called automatically when you exit 
.I nomadii
.RE

.B model        
.RS
displays Nomad II model and firmware information. Known models include:
.IR "Nomad II" ,
.IR "Nomad IIc" ,
.IR "Nomad II MG" .
.RE

.B owner [get]
.RS
return owner string
.RE

.B owner set <name>
.RS
set owner string to
.IR name ;
use double-quotes for multi-word names
.RE

.B internal
.RS
use internal memory for operations when both are present.
.I Format
requires
.I internal 
or 
.I external
to be set when both internal and external memory
is present.
.IR List ,
.IR send ,
.IR get ,
.IR playlist ,
.I memory
and
.IR count
are effected by this mode.
.RE

.B external
.RS
use external memory for operations when both are present.
.I Format
requires
.I internal 
or 
.I external
to be set when both internal and external memory
is present.
.IR List ,
.IR send ,
.IR get ,
.IR playlist ,
.I memory
and
.IR count
are effected by this mode.
.RE

.B both
.RS
use both internal and external memory for operations when both are present
(the default) - the
.I format
command is disallowed in this mode. When sending files in
this mode, the internal memory is tried first if there is room, then the
external memory. When sending multiple files or
a playlist, once a file is downloaded to the external memory for space reasons,
the rest of the items in the group or playlist are also downloaded to external
memory to preserve the order of the files/playlist.

.IR List ,
.IR send ,
.IR get ,
.IR playlist ,
.I memory
and
.IR count
are effected by this mode.
.RE

.B memory
.RS
display Nomad II memory usage:
Total/Used/Free in MB
.RE

.B count
.RS
display number of files in flash memory
.RE

.BR list ,
.B ls
.RS         
list the contents of the Nomad II flash memory
.RE

.B version
.RS      
display 
.I nomadii
version
.RE

.B format       
.RS
creates an empty FAT filesystem on the flash memory of the device.
.B WARNING
this operation erases the contents on the specified memory.
It prompts for a confirmation by default, unless
.I --force
is set. See
.I internal
and
.I external
if both internal and external memory are present. This command
cannot be used if 
.I both
mode is active.
.RE

.B presets
.RS
display list of Nomad II FM radio presets
.RE

.B presets set <n> <f1f2>
.RS
set radio preset number
.I n 
to frequency 
.IR f1.f2MHz .
After setting presets you must do a 
.I presets write 
to save the presets to the device.
.RE

.B presets read
.RS
refresh radio presets from Nomad II
.RE

.B presets write
.RS
saves radio presets to the Nomad II; use it after you make changes with 
.I "presets set"
.RE

.B clock [get]       
.RS
display date and time from Nomad II
.RE

.B clock set    
.RS
set Nomad II date and time to now
.RE

.B probe
.RS
rescan USB bus for a Nomad II
.RE

.B sync
.RS
synchronize all information with device. I.e. refetch
full details from the device.
If it succeeds you will see message: "Synced OK."
If it fails you get a warning; trying again may succeed.
A 
.I sync
is performed automatically after
.IR send ,
.IR get ,
.IR playlist ,
.IR delete
.RE

.BR send ,
.BR put ,
.B download <name> ...
.RS
copy one or more files
from computer to Nomad II.
File names can be an absolute or relative paths;
file names can have spaces only if you use double-quotes. 
MP3 files must have an .mp3 suffix
and NVF files must have an .nvf
suffix to be recognized by the player.
.RE

.B playlist <name> [<offset> [<number>]]
.RS
copy the files indicated in the playlist named
.I name
to Nomad II;
playlist formats tested: xmms & winamp.
Playlist file is a text file with a file name per line.
Lines starting with spaces or comments "#" are ignored.
If 
.I offset 
is present, start from that item in the playlist. If
.I number 
is present, only transfer that number of items from the playlist.
.RE

.BR get ,
.B upload all|<n> ...
.RS
copy files
with the specified numbers from Nomad II flash memory to computer or all files with the keyword
.IR all .
.I n 
must be a valid file number as showed on the output of the list command.
.RE

.B delete all|<n> ...
.RS
delete files
with the specified numbers from Nomad II flash memory or all files with the keyword
.IR all .
.I n 
must be a valid file number as showed on the output of the list command.
.SH COMMAND-LINE EXAMPLES
Format external flash memory without confirmation:
.RS

.B nomadii -x -F --format
.RE

List flash memory contents:
.RS

.B nomadii -l
.RE

Download a file to Nomad II without displaying progess:
.RS

.B nomadii -H -d demo.mp3
.RE

Set FM Preset number 10 to 93.90MHz
.RS

.B nomadii --freq 9390 --presets-set 10

.SH PERMISSIONS
By default, the ownership of the newly created device file
for the Nomad II is owned by root, without group/other read+write
permissions. This can be addressed by a number of techniques:

make nomadii 
.IR "setuid root" ;
on startup, 
.I nomadii
 detects it is running in this
mode and changes the owner of the device file to the
original user, and then immediately drops back to
the user ID of the original user (in order to minimize
the possibility of security issues)

use the 
.IR devuid ,
.I devgid
or
.I devmode
mount options for
.IR usbdevfs ,
either in your
.I /etc/fstab file
or located where the 
.I mount 
call for 
.I usbdevfs
is made
.RI ( /etc/rc.d/rc.sysinit 
on RedHat 7.x); 
for example use 
.I devmode=0666 
to open full access to all USB devices
(with the attendant insecureness this creates).
In a multi-user machine, setting up a group for
USB would be the more secure option using this method.

locate and install the
.I usb_perms
daemon

use the
.I hotplug
package; this subsystem has good support for
USB devices with user-level drivers such as this one with its 
.I usb.usermap 
launch method. The module launched by
.I hotplug
can be used to set ownerships or permissions
on the device file as soon as the Nomad II is
connected, and then even launch an
.I xterm
running a 
.I nomadii 
session. This is the
most delicate of these methods to set up - sample
instructions are given in the HOTPLUG file that
comes with the source distribution.

.SH FILES
.I /proc/bus/usb/devices
.RS
USB kernel dynamic device status as provided by usbdevfs. 
.RE
.I /proc/bus/usb/drivers
.RS
USB kernel dynamic drivers information as provided by usbdevfs.
.SH BUGS
Due to firmware bugs in most Nomad MP3 players, uploaded data can
contain duplicated data blocks - the software detects this and
autocorrects if it can. Further, deleting items through the USB
interface does not always seem to free all memory used by
the files.
.SH REPORTING BUGS
Report bugs to 
.I <nomadii-driver@lists.sourceforge.net>
.SH AUTHOR
Gabriel Gomiz 
.I <gomita@bblanca.com.ar>
.SH CONTRIBUTORS
Tom Mander, Ben Ross, Neil Hodge, Brendan Bouffler, Michal Ambroz,
Jacek Pliszka and Matthew Sanderson.
